xquery version "3.1";

(:~
 : This module provides the API endpoints for retrieving data from tei documents.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 8.1.0
 :)
module namespace kmg-data="https://mollenhauer-edition.de/ns/kmg-util";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "../conversions/selection.xqm";
import module namespace json = "https://mollenhauer-edition.de/ns/conversions/json" at "../conversions/json.xqm";
import module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref" at "../conversions/bibref.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "../kmg-util.xqm";
import module namespace rest="http://exquery.org/ns/restxq";
import module namespace functx="http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace lido="http://www.lido-schema.org";

(:~
 : Returns formatted data in the teiHeader of the requested document.
 :
 : @param $uri A valid TextGrid-URI without prefix
 : @return The requested data in JSON
 :)
declare
  %rest:GET
  %rest:path("/api/data/{$uri}")
  %output:method("json")
function kmg-data:get-data($uri as xs:string)
as map()*
{
    let $doc-path := $config:tei || $uri || ".xml"
    let $doc := doc($doc-path)//tei:teiHeader
    return
        kmg-data:format-data($doc)
};

(: ~
 : Formats relevant data about title, license and witnesses
 : in the teiHeader for display via the API.
 :
 : @param $doc a TEI-XML-document
 : @return A JSON object
 :)
declare function kmg-data:format-data($doc as node()*)
as map()*
{
    let $licence := map {"name": normalize-space($doc//tei:licence/text()), "url": string($doc//tei:licence/@target)}
    let $titles :=
        for $title in $doc//tei:title return
            map {string($title/@type): normalize-space($title/text())}
    let $witnesses :=
        for $witness in $doc//tei:sourceDesc/tei:listWit/tei:witness return
            map {
                string($witness/@xml:id): map
                {
                    "description": $witness/tei:desc/text(),
                    "type": if (contains($witness/tei:desc/@type, "Leittext")) then "main" else "version",
                    "year": functx:get-matches($witness/tei:desc, "\d{4}")[1]
                }
            }
    let $editors :=
        for $editor in $doc//tei:titleStmt/tei:editor return
            map {
                "forename": $editor/tei:name/tei:forename/text(),
                "surname": $editor/tei:name/tei:surname/text()
            }
    let $authors :=
        for $author in $doc//tei:titleStmt/tei:author return
            map {
                "forename": $author/tei:name/tei:forename/text(),
                "surname": $author/tei:name/tei:surname/text()
            }
    let $publisher :=
        for $publisher in $doc//tei:publicationStmt/tei:publisher return
            map {
                "name": $publisher/tei:name/text()
            }
    let $editionnr := $doc//tei:publicationStmt/tei:idno[@type='KMG']/text()
    let $output :=
        map {
            "title": map:merge($titles),
            "licence": $licence,
            "witnesses": map:merge($witnesses),
            "editors": $editors,
            "authors": $authors,
            "publisher": $publisher,
            "editionnr": $editionnr
        }
    return
        $output
};
